@@ -6,7 +6,6 @@ use base qw/Test::Builder::Module Exporter/;
 use Data::Dumper;
 use Carp;
 use Test::Builder;
-use Test::Deep qw(eq_deeply);
 
 our @EXPORT_OK = qw/&is_same_sql_bind &is_same_sql &is_same_bind
                     &eq_sql_bind &eq_sql &eq_bind 
@@ -177,7 +176,10 @@ sub eq_sql_bind {
 sub eq_bind {
   my ($bind_ref1, $bind_ref2) = @_;
 
-  return eq_deeply($bind_ref1, $bind_ref2);
+  local $Data::Dumper::Useqq = 1;
+  local $Data::Dumper::Sortkeys = 1;
+
+  return Dumper($bind_ref1) eq Dumper($bind_ref2);
 }
 
 sub eq_sql {
